import { isDefined, nextId, batch } from '@collectable/core';
import { emptyTree, thawTree, freezeTree, isTreeThawed, cloneTree, emptyHashMap, thawHashMap, freezeHashMap, isHashMapThawed, cloneHashMap } from './named-externals';
import { setItem } from './values';
import { isEqual, unwrap } from '../functions';
import { iteratePairs } from './iterate';
const SORTEDSET_TYPE = {
    type: Symbol('Collectable.SortedMap'),
    indexable: false,
    equals(other, set) {
        return isEqual(other, set);
    },
    unwrap(set) {
        return unwrap(true, set);
    },
    group(set) {
        return set._group;
    },
    owner(set) {
        return set._owner;
    }
};
export class SortedMapImpl {
    constructor(_owner, _group, _keyMap, _sortedValues, _compare, _select) {
        this._owner = _owner;
        this._group = _group;
        this._keyMap = _keyMap;
        this._sortedValues = _sortedValues;
        this._compare = _compare;
        this._select = _select;
    }
    get '@@type'() { return SORTEDSET_TYPE; }
    [Symbol.iterator]() {
        return iteratePairs(this);
    }
}
export function isSortedMap(arg) {
    return arg && arg['@@type'] === SORTEDSET_TYPE;
}
export function cloneSortedMap(mutable, set, clear = false) {
    var map;
    var tree;
    if (clear) {
        map = emptyHashMap(mutable);
        tree = emptyTree(mutable, set._compare);
    }
    else {
        map = mutable ? isHashMapThawed(set._keyMap) ? cloneHashMap(set._keyMap) : thawHashMap(set._keyMap) : freezeHashMap(set._keyMap);
        tree = mutable ? isTreeThawed(set._sortedValues) ? cloneTree(set._sortedValues) : thawTree(set._sortedValues) : freezeTree(set._sortedValues);
    }
    return new SortedMapImpl(batch.owner(mutable), nextId(), map, tree, set._compare, set._select);
}
export function cloneAsImmutable(set) {
    return cloneSortedMap(false, set);
}
export function cloneAsMutable(set) {
    return cloneSortedMap(true, set);
}
export function refreeze(set) {
    set._owner = 0;
    set._sortedValues._owner = 0;
    set._keyMap._owner = 0;
    return set;
}
export function createSet(mutable, values, compare, select) {
    var set = emptySet(true, compare, select);
    var map = set._keyMap;
    var tree = set._sortedValues;
    if (Array.isArray(values)) {
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            setItem(value[0], value[1], map, tree, select);
        }
    }
    else {
        var it = values[Symbol.iterator]();
        var current;
        while (!(current = it.next()).done) {
            var value = current.value;
            setItem(value[0], value[1], map, tree, select);
        }
    }
    return refreeze(set);
}
export function extractTree(set) {
    return set._sortedValues;
}
export function extractMap(set) {
    return set._keyMap;
}
const DEFAULT_COMPARATOR = (a, b) => a.index - b.index;
const COMPARATOR_CACHE = new WeakMap();
function createComparatorFn(compare) {
    var fn = COMPARATOR_CACHE.get(compare);
    return isDefined(fn) ? fn : (fn = function (a, b) {
        var n = compare(a, b);
        return n === 0 ? DEFAULT_COMPARATOR(a, b) : n;
    }, COMPARATOR_CACHE.set(compare, fn), fn);
}
export function emptySet(mutable = false, compare, select) {
    var comparator = isDefined(compare) ? createComparatorFn(compare) : DEFAULT_COMPARATOR;
    if (!mutable && comparator === DEFAULT_COMPARATOR) {
        return isDefined(EMPTY) ? EMPTY : (EMPTY = new SortedMapImpl(0, 0, emptyHashMap(), emptyTree(), comparator, void 0));
    }
    var map = emptyHashMap(mutable);
    var tree = emptyTree(mutable, comparator);
    return new SortedMapImpl(batch.owner(mutable), nextId(), map, tree, comparator, select);
}
var EMPTY;

//# sourceMappingURL=SortedMap.js.map
